----------------------------------------------------------------------------------
-- Company: Hochschule RheinMain
-- Engineer: Steffen Reith <Steffen.Reith@hs-rm.de>
-- 
-- create Date:    16:10:58 04/17/2014 
-- Design Name: 
-- Module Name:    GrayCnt - Behavioral 
-- Project Name:   GrayCnt - A simple counter with gray encoding
--
-- Revision: $Id: GrayCnt.vhd 222 2014-06-10 09:27:52Z streit $
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;  
use unisim.vcomponents.all;

entity GrayCnt is

  generic(width : integer := 8);
  
  port (clk    : in  std_logic;
        reset  : in  std_logic;
        enable : in  std_logic;
        gval   : out std_logic_vector(width - 1 downto 0));

end GrayCnt;

architecture Native of GrayCnt is

  signal hBit_reg  : std_logic; -- Hold the toggling bit
  signal hBit_next : std_logic;

  signal val_reg  : std_logic_vector(width - 1 downto 0); -- Generated gray code
  signal val_next : std_logic_vector(width - 1 downto 0);
  
begin

  state_logic : process (clk, reset)
  begin

    -- Check for rising edge
    if (rising_edge(clk)) then

      -- Synchron reset
      if(reset = '1') then

        -- Set helper bit to 0
        hBit_reg <= '1';

        -- Init gray code to last code word
        val_reg <= (val_reg'high => '1', others => '0');

      else 
      
        -- Check if counter is enabled
        if (enable = '1') then
        
          -- Set bit on rising edge 
          hBit_reg <= hBit_next;

          -- Set gray code value at rising edge
          val_reg <= val_next;
          
        end if;

      end if;
  
    end if;

  end process;

  -- next state logic for the gray code
  next_state : process(val_reg, hBit_reg)

    variable tmp : std_logic;
    
  begin

    -- Next state logic for the toggling bit
    hBit_next <= not hBit_reg;

    -- Handle the LSB
    val_next(val_next'low) <= not(val_reg(val_reg'low) xor (hBit_reg)); 
   
    -- Iterate over the equations for the inner bits
    for i in 1 to width - 2 loop

      -- Init the temporal bit
      tmp := '1';
      
      -- Calculate the part which checks for 0^*
      for j in 0 to i - 2 loop

        -- This equation checks the 0^* pattern
        tmp := tmp and not(val_reg(j));

      end loop;

      -- Now tmp checks 10^*
      tmp := val_reg(i - 1) and tmp and hBit_reg;

      -- Full ith equation
      val_next(i) <= val_reg(i) xor tmp;
      
    end loop;

    -- Handle the MSB
    tmp := '1';
    for j in 0 to width - 3 loop

      -- This equation checks 0^*
      tmp := tmp and not(val_reg(j));

    end loop;

    -- Full equation for the MSB
    val_next(val_next'high) <= val_reg(val_reg'high) xor (tmp and hBit_reg);
      
  end process;

  -- Output logic
  gval <= val_reg;
  
end architecture;
