----------------------------------------------------------------------------------
-- Company: Hochschule RheinMain
-- Engineer: Steffen Reith <Steffen.Reith@hs-rm.de>
-- 
-- Create Date:    16:20:58 06/09/2014 
-- Design Name: 
-- Module Name:    GrayCnt_TB - Behavioral 
-- Project Name:   GrayCnt - A simple counter with gray encoding
--
-- Revision: $Id: aFifo_TB.vhd 222 2014-06-10 09:27:52Z streit $
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library unisim;
use unisim.vcomponents.all;

entity testbench is
end testbench;

architecture behavior of testbench is

  constant dWidth : integer := 8;
  constant aWidth : integer := 5;

  -- Clock period definitions
  constant wClk_period : time := 11 ns;
  constant rClk_period : time := 31 ns;

  -- Signals for the UUT
  signal clear : std_logic;

  signal readData  : std_logic_vector(dWidth - 1 downto 0);
  signal writeData : std_logic_vector(dWidth - 1 downto 0);

  signal isEmpty : std_logic;
  signal isFull  : std_logic;

  signal readEnable  : std_logic;
  signal writeEnable : std_logic;

  signal rClk : std_logic;
  signal wClk : std_logic;
  
begin

  uut : entity work.aFifo(Behavioral)
    generic map (dWidth => 8,
                 aWidth => 5)
    port map (clear       => clear,
              readData    => readData,
              isEmpty     => isEmpty,
              readEnable  => readEnable,
              rClk        => rClk,
              writeData   => writeData,
              isFull      => isFull,
              writeEnable => writeEnable,
              wClk        => wClk);

  -- Write clock process definitions
  wClk_process : process
  begin

    wClk <= '0';
    wait for wClk_period / 2;

    wClk <= '1';
    wait for wClk_period / 2;

  end process;

  -- Read clock process definitions
  rClk_process : process
  begin

    rClk <= '0';
    wait for rClk_period / 2;

    rClk <= '1';
    wait for rClk_period / 2;

  end process;

  writer : process

    variable seed1   : positive;
    variable seed2   : positive;
    variable rand    : real;
    variable randInt : integer;
    
  begin

    -- Check if fifo is not full
    if (isFull = '0') then

      -- Generate and (store) a random number
      uniform(seed1, seed2, rand);
      randInt   := integer(trunc(rand * real(2**dwidth)));
      writeData <= std_logic_vector(to_unsigned(randInt, writeData'length));

      -- Enable writing to the fifo and wait for rising edge
      writeEnable <= '1';

      -- Wait for next rising edge
      wait until rising_edge(wClk);

      -- Write until falling edge (hence we have at least one full clock)
      wait until falling_edge(wClk);
      
    end if;

    -- Don't write to fifo
    writeEnable <= '0';

    -- Wait a random time
    uniform(seed1, seed2, rand);
    wait for rand * 10.0 * wClk_period + 3ns;

  end process;

  reader : process

    variable seed1   : positive;
    variable seed2   : positive;
    variable rand    : real;
    variable randInt : integer;
    
  begin

    -- Check if fifo is not full
    if (isEmpty = '0') then

      -- Enable writing to the fifo and wait for rising edge
      readEnable <= '1';

      -- Wait for next rising edge
      wait until rising_edge(rClk);

      -- Write until falling edge (hence we have at least one full clock)
      wait until falling_edge(wClk);

    end if;

    -- Do not read anymore
    readEnable <= '0';

    -- Wait a random time
    uniform(seed1, seed2, rand);
    wait for rand * 10.0 * rClk_period + 3ns;

  end process;

  tb : process
  begin

    clear <= '1';
    wait for 100 ns;  -- wait until global set/reset completes
    clear <= '0';

    wait;                               -- wait forever

  end process;

end architecture;
